"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var moment = require("moment");
var jwt = require("jsonwebtoken");
var log4js = require("log4js");
var jwt_config_1 = require("../config/jwt.config");
var mssql_service_1 = require("../db/mssql.service");
var redis_service_1 = require("../db/redis.service");
var user_service_1 = require("./user.service");
var role_service_1 = require("./role.service");
var promises_shared_1 = require("../shared/promises.shared");
var logger = log4js.getLogger('ServerScripts');
function AuthService(app) {
    app.post('/auth/login', function (req, res, next) {
        logger.info('[AuthService::login] Received ' + req.method + ' method on \'' + req.path + '\'');
        var nowUtc = moment().utc().toDate();
        var credential = req.body.userId;
        var credentials = credential.split('@');
        var userId = credentials.length > 0 ? credentials[0] : '';
        var password = req.body.password;
        var vcc = res.locals.vcc;
        var session;
        var user;
        var role;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!userId || !password)
                throw 'Missing userId and password input parameters';
            userId = userId.toLowerCase();
            return mssql_service_1.MSSql.ValidateUserLogin(userId, password);
        }).then(function (resultValidateUserLogin) {
            if (!resultValidateUserLogin)
                throw 'El usuario ó contraseña no son correctos';
            session = jwt.sign({ userId: userId }, jwt_config_1.JWT_SECRET, { expiresIn: jwt_config_1.JWT_EXPIRATION_TIME });
            return redis_service_1.Redis.SaveSession(userId, session);
        }).then(function (resultSaveSession) {
            if (!resultSaveSession)
                throw 'Cannot save session cache on Redis';
            return user_service_1.GetUser(userId);
        }).then(function (resultUser) {
            if (!resultUser)
                throw 'Cannot get user data';
            user = resultUser;
            return mssql_service_1.MSSql.InsertUserHistory(userId, user.role, 'Login', nowUtc, session);
        }).then(function (resultInsertUserHistory) {
            if (!resultInsertUserHistory)
                throw 'Cannot save session login history on database';
            return role_service_1.GetRole(user.role);
        }).then(function (resultRole) {
            if (!resultRole)
                throw 'Cannot get user role data';
            role = resultRole;
            if (!session || !user || !role)
                throw 'Se produjo un error al iniciar la sesión';
            var data = {
                session: session,
                loggedUser: user,
                loggedUserRole: role
            };
            res.cookie('Session', session, { httpOnly: true });
            logger.info('[AuthService::/auth/login] Se inicio sesion ' + session + ' para el usuario ' + userId + '@');
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: data });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'Login failed');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('[AuthService::/auth/login] Error logging for user ' + userId + ' : ' + errorMsg);
            if (res.headersSent)
                return;
            res.status(401).json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/auth/logout', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var userId = res.locals.userId;
        var user = res.locals.user;
        var nowUtc = moment().utc().toDate();
        var session = req.body.session || res.locals.session || '';
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return redis_service_1.Redis.DeleteSession(userId);
        }).then(function (result) {
            return redis_service_1.Redis.DeleteUserById(userId);
        }).then(function (result) {
            return mssql_service_1.MSSql.InsertUserHistory(userId, user.role, 'Logout', nowUtc, session);
        }).then(function (result) {
            res.clearCookie('Session');
            logger.info('Se finalizo sesion ' + session + ' para el usuario ' + userId);
            if (res.headersSent)
                return;
            res.json({ status: result, description: (result ? 'OK' : 'Logout failed'), data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'Logout failed');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al finalizar sesion para el usuario ' + userId + ' : ' + errorMsg);
            if (res.headersSent)
                return;
            res.status(401).json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/auth/get_user_session_data', function (req, res, next) {
        logger.info('[AuthService::get_user_session_data] Received ' + req.method + ' method on \'' + req.path + '\'');
        var userId = res.locals.userId;
        userId = (userId || '').toLowerCase();
        var session;
        var user;
        var role;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return redis_service_1.Redis.GetSession(userId);
        }).then(function (resultSession) {
            if (!resultSession)
                throw 'Cannot get user session';
            session = resultSession;
            return user_service_1.GetUser(userId);
        }).then(function (resultUser) {
            if (!resultUser)
                throw 'Cannot get user data';
            user = resultUser;
            return role_service_1.GetRole(user.role);
        }).then(function (resultRole) {
            if (!resultRole)
                throw 'Cannot get user role data';
            role = resultRole;
            if (!session || !user || !role)
                throw 'Se produjo un error al iniciar la sesión';
            var data = {
                session: session,
                loggedUser: user,
                loggedUserRole: role
            };
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: data });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.AuthService = AuthService;
