"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var url = require("url");
var jwt = require("jsonwebtoken");
var log4js = require("log4js");
var jwt_config_1 = require("../config/jwt.config");
var domain_model_1 = require("../models/domain.model");
var mssql_service_1 = require("../db/mssql.service");
var redis_service_1 = require("../db/redis.service");
var promises_shared_1 = require("../shared/promises.shared");
var logger = log4js.getLogger('ServerScripts');
function GetRequestData(req, res, next) {
    var host = req.headers['host'] || '';
    if (host.match(/^http:\/\//))
        host = host.substring(7);
    if (host.match(/^https:\/\//))
        host = host.substring(8);
    if (host.match(/^www\./))
        host = host.substring(4);
    if (!/http:/.test(host) && !/https:/.test(host))
        host = 'http://' + host;
    res.locals.protocol = req.protocol;
    res.locals.host = req.headers['host'];
    res.locals.pageUrl = req.protocol + '://' + req.get('host') + req.originalUrl;
    res.locals.pageReferrer = req.headers['referer'];
    res.locals.origin = req.headers['origin'];
    var requestDomain = url.parse(host).hostname;
    res.locals.requestDomain = requestDomain;
    promises_shared_1.DummyPromise()
        .then(function (result) {
        return next();
    }).catch(function (err) {
        var errMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
        logger.error('Error al procesar \'' + req.path + '\': ' + errMsg);
        if (res.headersSent)
            return;
        res.status(401).json({ description: 'Undefined domain (' + requestDomain + ')', error: 'VCC no válido' });
    });
}
exports.GetRequestData = GetRequestData;
function ValidateDomainRedirect(req, res, next) {
    var domain = res.locals.domain;
    if (domain && domain.isContentDomain && domain.contentDefaultRedirect && domain.contentDefaultRedirect !== req.url && !req.url.match('^\/(api|auth|public|internal|popup|builder|payments|distributed|static|template|assets|favicon)(.*)$')) {
        res.redirect(domain.contentDefaultRedirect);
    }
    else {
        next();
    }
}
exports.ValidateDomainRedirect = ValidateDomainRedirect;
function GetDomainFromRequestHost(requestHost) {
    var host = requestHost || '';
    if (host.match(/^http:\/\//))
        host = host.substring(7);
    if (host.match(/^https:\/\//))
        host = host.substring(8);
    if (host.match(/^www\./))
        host = host.substring(4);
    if (!/http:/.test(host) && !/https:/.test(host))
        host = 'http://' + host;
    var requestDomain = url.parse(host).hostname;
    return requestDomain;
}
exports.GetDomainFromRequestHost = GetDomainFromRequestHost;
function GetRequestInstance(requestHost) {
    var host = requestHost || '';
    if (host.match(/^http:\/\//))
        host = host.substring(7);
    if (host.match(/^https:\/\//))
        host = host.substring(8);
    if (host.match(/^www\./))
        host = host.substring(4);
    if (!/http:/.test(host) && !/https:/.test(host))
        host = 'http://' + host;
    var domain = url.parse(host).hostname;
    return GetInstanceByDomain(domain)
        .then(function (vcc) {
        if (vcc) {
            return Promise.resolve(vcc);
        }
        else {
            logger.warn('El dominio \'' + domain + '\' no esta asociado a ninguna instancia. Se usara instancia por defecto.');
            return Promise.resolve('default');
        }
    }).catch(function (err) {
        var errMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
        logger.error('Error al obtener instancia asociada al request: ' + errMsg);
        return Promise.resolve('default');
    });
}
exports.GetRequestInstance = GetRequestInstance;
function GetRequestUser(authorizationHeader) {
    var session = authorizationHeader || '';
    var userId;
    try {
        session = session.replace(/^Bearer /i, '');
        if (session) {
            var decoded = jwt.verify(session, jwt_config_1.JWT_SECRET);
            if (decoded.userId)
                userId = decoded.userId;
        }
    }
    catch (err) {
        var errMsg = (typeof err === 'string' ? err : err.message || err.description || '');
        logger.error('Error al decodificar datos de la sesion: ' + errMsg);
    }
    return userId;
}
exports.GetRequestUser = GetRequestUser;
function GetInstanceByDomain(domain) {
    var cacheUpdateNeeded = false;
    return redis_service_1.Redis.GetInstanceByDomain(domain)
        .then(function (redisInstance) {
        if (redisInstance || redisInstance === '') {
            return Promise.resolve(redisInstance);
        }
        else {
            logger.info('El dato de GetInstanceByDomain para \'' + domain + '\' no se encuentra en cache. se consultara a base de datos.');
            cacheUpdateNeeded = true;
            return mssql_service_1.MSSql.GetInstanceByDomain(domain);
        }
    }).then(function (vcc) {
        if (cacheUpdateNeeded)
            redis_service_1.Redis.SaveInstanceByDomain(domain, vcc || '');
        if (vcc || vcc === '') {
            return Promise.resolve(vcc);
        }
        else {
            throw 'Se consulto dato de GetInstanceByDomain para \'' + domain + '\' que no se encuentra definido en base de datos.';
        }
    }).catch(function (err) {
        var errMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
        logger.error('Error al ejecutar GetInstanceByDomain: ' + errMsg);
    });
}
function GetDomain(vcc, domain) {
    var cacheUpdateNeeded = false;
    var defaultDomain = new domain_model_1.Domain(domain, domain, null, false, '80', null, false, false, null, false, null, null, null, null, null, null);
    return redis_service_1.Redis.GetDomain(vcc, domain)
        .then(function (redisDomain) {
        if (redisDomain || JSON.stringify(redisDomain) === JSON.stringify(defaultDomain)) {
            return Promise.resolve(redisDomain);
        }
        else {
            logger.info('El dato de GetDomain para \'' + domain + '\' no se encuentra en cache. se consultara a base de datos.');
            cacheUpdateNeeded = true;
            return mssql_service_1.MSSql.GetDomain(vcc, domain);
        }
    }).then(function (resultDomain) {
        if (cacheUpdateNeeded)
            redis_service_1.Redis.SaveDomain(vcc, resultDomain || defaultDomain);
        if (resultDomain || JSON.stringify(resultDomain) === JSON.stringify(defaultDomain)) {
            return Promise.resolve(resultDomain);
        }
        else {
            throw 'Se consulto dato de GetDomain para \'' + domain + '\' que no se encuentra definido en base de datos.';
        }
    }).catch(function (err) {
        var errMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
        logger.error('Error al ejecutar GetDomain: ' + errMsg);
    });
}
